function [path Vpath]=procedure(costs,T,iceberg)

M=size(costs,1);
K=size(cell2mat(costs(M)),2);
N=size(cell2mat(costs(2)),1);

% Store the results in cell arrays:
Vvalues=cell(M,1);
locations=cell(M,1);



% Stage 1

costs1=cell2mat(costs(M));

% Now the choice has to be made for every state of the world (production
% location on the next stage)

costs13d=repmat(costs1,1,1,K);


% Let's start with specific trade costs and later upgrade this code to
% allow for choosing

% First, we'll need a decision matrix. It'll be an indicator matrix of the
% same dimensions as costs and will reflect which elements of costs matrix
% are active


% Tind is just a diagonal matrix that indicates that trade costs are
% (in)active. I want it to align with the dimensions of costs, so I repeat
% it in part and stage dimensions


Tind=permute(repmat(eye(K),1,1,N^(M-1)),[3 1 2]);

% First parts. Minimum finds the value of the value function V and the
% index of the country to be chosen under any state of the world
[V,country_index]=min(costs13d+costs13d.*T.*(1-Tind).*iceberg+T.*(1-Tind).*(1-iceberg),[],2);

V=squeeze(V);
Vvalues{1}=V;
country_index=squeeze(country_index);
locations{1}=country_index;

% So here I add up value functions values for parts that are to be
% assembled at the next stage. Further is just an algebra: I find the
% cumulative sum, keep every N-th element, then add up a vector of zeros on
% top of that and find the first difference to recover the sum for each
% node only
check01=cumsum(V,1);
temp1=check01(N:N:end,:);
temp2=[zeros(1,K); temp1];
Vcum1=diff(temp2);

% Stage i:

for i=2:M-1
costs0=cell2mat(costs(M+1-i));
costs3d=repmat(costs0,1,1,K);



Tind=permute(repmat(eye(K),1,1,N^(M-i)),[3 1 2]);

[V,country_index]=min(Vcum1+costs3d+(Vcum1+costs3d).*T.*(1-Tind).*iceberg+T.*(1-Tind).*(1-iceberg),[],2);

V=squeeze(V);
Vvalues{i}=V;
country_index=squeeze(country_index);
locations{i}=country_index;

check01=cumsum(V,1);
temp1=check01(N:N:end,:);
temp2=[zeros(1,K); temp1];
Vcum1=diff(temp2);

end

% At the last stage there's no need to sum up value functions for the next
% stage, this is the only difference

% Stage M
% At the final stage the decision is not conditional on the location at the next stage.
% Trade costs are already accounted in the previous stage’s value function

costs0=cell2mat(costs(1));


[V,country_index]=min(Vcum1+costs0,[],2);

V=squeeze(V);
Vvalues{M}=V;
country_index=squeeze(country_index);
locations{M}=country_index;


% Now when we know the production location at the final stage, we can
% construct the optimal path and corresponding values of the value function
path=cell(M,1);
Vpath=cell(M,1);
path{M}=locations{M};
Vpath{M}=Vvalues{M};

for i=1:M-1
    loc0=cell2mat(locations(M-i));
    loc1=cell2mat(path(M+1-i));
    val0=cell2mat(Vvalues(M-i));
    for j=1:N^(i-1)
        path0(1+(j-1)*N:j*N)=loc0(1+(j-1)*N:j*N,loc1(j));
        Vpath0(1+(j-1)*N:j*N)=val0(1+(j-1)*N:j*N,loc1(j));
        
    end

    path{M-i}=path0;
    Vpath{M-i}=Vpath0;
    
end

end
